/*
 * Copyright (C) 2003-2004 Imendio AB
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <glib.h>
#include <string.h>
#include <stdlib.h>
#include <loudmouth/loudmouth.h>
#ifdef __WIN32__
#include <winsock2.h>
#endif
 
typedef struct {
    gchar		*server;
    gchar		*name;
    gchar		*passwd;
    LmConnection	*connection;
    GMainLoop		*main_loop;
    int			 return_value;
} UserInfo;

static void
loudgame_quit (UserInfo *info, int return_value)
{
    GError *error;

    info->return_value = return_value;

    if (! lm_connection_close (info->connection, &error))
	g_print ("An error occurred during lm_connection_close: %s\n",
		 error->message);

    lm_connection_unref (info->connection);

    g_main_loop_quit (info->main_loop);
}
 
static void
authentication_cb (LmConnection *connection, gboolean result, gpointer closure)
{
    LmMessage *m;
    UserInfo *info = closure;

    if (! result) {
	g_print ("Authentication for %s failed\n", info->name);
	loudgame_quit (info, 1);
	return;
    }
                 
    m = lm_message_new_with_sub_type (NULL,
				      LM_MESSAGE_TYPE_PRESENCE,
				      LM_MESSAGE_SUB_TYPE_AVAILABLE);

    lm_connection_send (connection, m, NULL);
    lm_message_unref (m);
}
 
static void
connection_open_cb (LmConnection *connection, gboolean result, UserInfo *info)
{
    lm_connection_authenticate (connection,
				info->name, info->passwd, "TestLM",
				authentication_cb, info, FALSE,  NULL);
}

static void
send_reply (LmConnection	*connection,
	    const char		*peer,
	    const char		*message,
	    UserInfo		*info)
{
    LmMessage *reply;
    gboolean result;
    GError *error = NULL;

    reply = lm_message_new (peer, LM_MESSAGE_TYPE_MESSAGE);

    lm_message_node_add_child (reply->node, "body", message);

    result = lm_connection_send (connection, reply, &error);
    lm_message_unref (reply);

    if (! result) {
	g_error ("lm_connection_send failed: error->message");
	loudgame_quit (info, 1);
    }
}

static void
handle_command (LmConnection	*connection,
		const char	*peer,
		const char	*command,
		UserInfo	*info)
{
    char *error;

    if (strcmp (command, "quit") == 0) {
	loudgame_quit (info, 0);
	return;
    }

    error = g_strdup_printf ("Unknown command: '%s'", command);
    send_reply (connection, peer, error, info);
    free (error);
}
 
static LmHandlerResult
handle_messages (LmMessageHandler *handler,
                 LmConnection     *connection,
                 LmMessage        *m,
                 gpointer          closure)
{
    UserInfo *info = closure;
    LmMessageNode *body;
    const char *peer;
    const char *body_str;

    peer = lm_message_node_get_attribute (m->node, "from");

    body = lm_message_node_get_child (m->node, "body");
    if (body) {
	body_str = lm_message_node_get_value (body);

	if (body_str && body_str[0] == '%')
	    handle_command (connection, peer, body_str + 1, info);
	else
	    send_reply (connection, peer, body_str, info);
    }
	
    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
}

static gboolean
make_connection (gpointer closure)
{
    UserInfo	     *info;
    LmMessageHandler *handler;
    gchar            *jid;
    GError	     *error;

    info = closure;
                                                                                
    info->connection = lm_connection_new (info->server);

    jid = g_strdup_printf ("%s@%s", info->name, info->server);
    lm_connection_set_jid (info->connection, jid);
    g_free (jid);

    handler = lm_message_handler_new (handle_messages, info, NULL);
    lm_connection_register_message_handler (info->connection,
					    handler,
					    LM_MESSAGE_TYPE_MESSAGE,
					    LM_HANDLER_PRIORITY_NORMAL);
                                                                                
    lm_message_handler_unref (handler);
                                                                                
    if (! lm_connection_open (info->connection,
			      (LmResultFunction) connection_open_cb,
			      info, NULL, &error))
    {
	g_print ("Opening connection failed: %s\n", error->message);
	loudgame_quit (info, 1);
    }

    /* Return false to not schedule another call. */
    return 0;
}

int
main (int argc, char **argv)
{
    UserInfo          info;

    if (argc != 4) {
	g_print ("Usage: %s <server> <username> <password>\n", argv[0]);
	return 1;
    }

    info.server = argv[1];
    info.name = argv[2];
    info.passwd = argv[3];

    info.connection = NULL;

    info.main_loop = g_main_loop_new (NULL, FALSE);

    g_idle_add (make_connection, &info);

    g_main_loop_run (info.main_loop);

    g_main_loop_unref (info.main_loop);
                                                                                
    return info.return_value;
}
